<?php

return [
    /**
     * デバッグモード
     * Twigのデバッグモードを設定します
     * 有効：1, 無効：0
     * 本番環境では有効にしないでください。
     */
    'debug' => 0,

    /**
     * エラー詳細表示
     * 有効：1, 無効：0
     * 画面にエラー詳細を表示するかどうかを設定します。
     * 本番環境では有効にしないでください。
     */
    'displayErrorDetails' => 0,

    /**
     * エラーログ
     * 有効：1, 無効：0
     * エラーログを記録するかどうかを設定します。これを有効にすると、管理画面のログ機能で内容を確認できます。
     */
    'logErrors' => 1,

    /**
     * エラー詳細ログ
     * 有効：1, 無効：0
     * 有効にすると、PHPのシステムロガーに詳細なログが送られます。
     * 詳細ログを記録する場合は、logErrorsも有効にしてください。
     */
    'logErrorDetails' => 0,

    /**
     * タイムゾーン
     */
    'timezone' => 'Asia/Tokyo',

    /**
     * ロケール
     */
    'locale' => 'ja_JP.UTF-8',

    /**
     * PHP設定
     */
    'ini_set' => [
        'memory_limit' => '256M',
    ],

    /**
     * ログ設定
     */
    'logger' => [
        // 期間は 無指定, yearly, monthly, daily を指定できます
        'period' => 'monthly',
        // ログレベルは DEBUG, INFO, NOTICE, WARNING, ERROR, CRITICAL, ALERT, EMERGENCY を指定できます
        'level' => \Monolog\Logger::INFO,
    ],

    /**
     * ファイルシステム設定
     */
    'filesystem' => [
        // パーミッションは8進数で指定する必要があるため、先頭に 0 を置いてください。例：0755
//        'file_permission' =>  0775,
//        'dir_permission' =>  0775,
//        'file_group' =>  'work',
//        'dir_group' =>  'work',
    ],

    /**
     * ドキュメントルート
     */
    //'document_root' => '/var/www/html',

    /**
     * テーマディレクトリ
     */
    'themes_dir' => 'themes',

    /**
     * 拡張子別アイコン
     */
    'icons' => [
        'picture-o' => ['jpeg', 'jpg', 'png', 'gif', 'webp', 'svg'],
        'file-code-o' => ['twig', 'js'],
        'file-pdf-o' => ['pdf'],
        'html5' => ['html'],
        'css3' => ['css'],
        'file-text-o' => ['txt'],
        'file-movie-o' => ['m4a', 'avi', 'mov', 'wmv', 'flv', 'mpg', 'mp4'],
        'file-audio-o' => ['mp3', 'wav', 'wma'],
        'file-excel-o' => ['xls', 'xlsx'],
        'file-word-o' => ['doc', 'docx'],
    ],

    /**
     * ページネーション設定
     */
    'pagination' => [
        // 1ページ辺りのページ数
        'per_page' => 20,
        // リンクの中央の数
        'mid_size' => 7,
        // リンクの両端の数
        'end_size' => 1,
        // 現在ページに適用するクラス
        'current_class' => 'active',
        // リンクに適用するクラス
        'link_class' => '',
        // リンクを省略した要素に適用するクラス
        'ellipsis_class' => 'disabled',
        // リンクを省略したことを表す文字列
        'ellipsis_text' => '...',
    ],

    /**
     * 管理画面日付フォーマット
     */
    'date_admin' => 'Y/m/d H:i',

    /**
     * 日付フォーマット（ロング）
     */
    'date_long' => 'Y/m/d H:i',

    /**
     * 日付フォーマット（ショート）
     */
    'date_short' => 'Y/m/d',

    /**
     * ローカル日付フォーマット（ロング）
     */
    'date_local_long' => '%Y年%m月%d日（%a） %H時%M分',

    /**
     * ローカル日付フォーマット（ショート）
     */
    'date_local_short' => '%Y年%m月%d日（%a）',
];
