<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin;

use Slim\App;
use MicroEngine\Admin\Middleware\AuthMiddleware;
use Slim\Interfaces\RouteCollectorProxyInterface;
use MicroEngine\Core\Middleware\DBTransactionMiddleware;
use MicroEngine\Install\Middleware\SchemaVersionMiddleware;

class AdminRoute
{
    /**
     * @param App $app
     */
    public function register(App $app)
    {
        $c = $app->getContainer();

        $app->group('/admin', function (RouteCollectorProxyInterface $group) {
            // ホーム
            $group->get('', 'homeController:index')->setName('admin.home.index');
            $group->get('/portlet', 'homeController:portlet')->setName('admin.home.portlet');
            $group->put('/portlet', 'homeController:updatePortlet')->setName('admin.home.updatePortlet');
            $group->get('/blank', 'homeController:blank')->setName('admin.home.blank');
            // ユーザー
            $group->group('/user', function (RouteCollectorProxyInterface $group) {
                $group->get('[/page/{page_no:[0-9]+}]', 'userController:index')->setName('admin.user.index');
                $group->get('/create', 'userController:create')->setName('admin.user.create');
                $group->post('', 'userController:store')->setName('admin.user.store');
                $group->get('/{id}/edit', 'userController:edit')->setName('admin.user.edit');
                $group->put('/{id}', 'userController:update')->setName('admin.user.update');
                $group->put('/unlock/{id}', 'userController:unlock')->setName('admin.user.unlock');
                $group->delete('/{id}', 'userController:destroy')->setName('admin.user.destroy');
            });
            // ブックマーク
            $group->group('/bookmark', function (RouteCollectorProxyInterface $group) {
                $group->get('', 'bookmarkController:index')->setName('admin.bookmark.index');
                $group->map(['GET', 'POST'], '/edit', 'bookmarkController:edit')->setName('admin.bookmark.edit');
                $group->put('', 'bookmarkController:update')->setName('admin.bookmark.update');
            });
            // 設定
            $group->group('/config', function (RouteCollectorProxyInterface $group) {
                $group->get('', 'configController:index')->setName('admin.config.index');
                $group->get('/{filename}/edit', 'configController:edit')->setName('admin.config.edit');
                $group->put('/{filename}', 'configController:update')->setName('admin.config.update');
            });
            // ライセンス
            $group->group('/license', function (RouteCollectorProxyInterface $group) {
                $group->get('', 'licenseController:index')->setName('admin.license.index');
                $group->get('/register', 'licenseController:register')->setName('admin.license.register');
                $group->post('', 'licenseController:store')->setName('admin.license.store');
            });
            // ログ
            $group->get('/log/viewer[/{filename}]', 'logController:viewer')->setName('admin.log.viewer');
            // システム情報
            $group->group('/system', function (RouteCollectorProxyInterface $group) {
                $group->get('', 'systemController:index')->setName('admin.system.index');
                $group->get('/phpinfo', 'systemController:phpinfo')->setName('admin.system.phpinfo');
                $group->get('/libs', 'systemController:libs')->setName('admin.system.libs');
            });
        })->add(new AuthMiddleware($c))->add(new SchemaVersionMiddleware($c))->add(new DBTransactionMiddleware($c));

        // ログイン・ログアウト
        $app->group('/admin/auth', function (RouteCollectorProxyInterface $group) {
            $group->get('/login', 'authController:login')->setName('admin.auth.login');
            $group->post('/login', 'authController:attemptLogin');
            $group->post('/logout', 'authController:logout')->setName('admin.auth.logout');
        })->add(new SchemaVersionMiddleware($c))->add(new DBTransactionMiddleware($c));
    }
}
