<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin;

use MicroEngine\Admin\Auth\Auth;
use Psr\Container\ContainerInterface;
use MicroEngine\Admin\Views\AdminExtension;
use MicroEngine\Core\Support\ServiceProvider;
use MicroEngine\Admin\Managers\{
    UserManager, MeSystemManager
};
use MicroEngine\Admin\Services\{
    ConfigService, Log, Portlet
};
use MicroEngine\Admin\Controllers\{
    LicenseController,
    LogController,
    AuthController,
    SystemController,
    UserController,
    HomeController,
    ConfigController,
    BookmarkController
};

class AdminServiceProvider extends ServiceProvider
{
    /**
     * サービス登録
     */
    public function register()
    {
        $c = $this->c;

        $c->set('auth', function (ContainerInterface $c) {
            return new Auth($c);
        });
        $c->set('meSystemManager', function (ContainerInterface $c) {
            return new MeSystemManager($c);
        });
        $c->set('userManager', function (ContainerInterface $c) {
            return new UserManager($c);
        });
        $c->set('portlet', function (ContainerInterface $c) {
            return new Portlet($c);
        });
        $c->set('log', function (ContainerInterface $c) {
            return new Log($c);
        });
        $c->set('configService', function (ContainerInterface $c) {
            return new ConfigService($c);
        });
        $c->set('authController', function () {
            return new AuthController($this->app);
        });
        $c->set('homeController', function () {
            return new HomeController($this->app);
        });
        $c->set('userController', function () {
            return new UserController($this->app);
        });
        $c->set('bookmarkController', function () {
            return new BookmarkController($this->app);
        });
        $c->set('configController', function () {
            return new ConfigController($this->app);
        });
        $c->set('licenseController', function () {
            return new LicenseController($this->app);
        });
        $c->set('logController', function () {
            return new LogController($this->app);
        });
        $c->set('systemController', function () {
            return new SystemController($this->app);
        });
    }

    /**
     * サービス起動
     */
    public function boot()
    {
        // Twigエクステンションを登録
        $this->c->get('view')->addExtension(new AdminExtension($this->c));

        // ルートを登録
        $route = new AdminRoute();
        $route->register($this->app);
    }
}
