<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Controllers;

use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use MicroEngine\Admin\Auth\Auth;
use MicroEngine\Admin\Forms\LoginForm;
use Psr\Http\Message\ResponseInterface;
use Rakit\Validation\RuleQuashException;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;

class AuthController extends Controller
{
    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function login(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new LoginForm();

        return $this->view->render(
            $response,
            'admin/views/auth/login.twig',
            compact('form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     * @throws RuleQuashException
     */
    public function attemptLogin(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new LoginForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーションチェック
        if (!$form->validate()) {
            return $this->view->render(
                $response,
                'admin/views/auth/login.twig',
                compact('form')
            );
        }

        /** @var Auth $auth */
        $auth = $this->c->get('auth');
        // ログイン認証チェック
        if ($auth->attempt($form->get('login_id'), $form->get('password'))) {
            // 認証成功
            $redirectTo = $auth->getRedirectTo();
            if (isset($redirectTo) && strlen($redirectTo)) {
                return $response->withHeader('location', $redirectTo)->withStatus(302);
            }
            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.home.index'))->withStatus(302);
        } else {
            // 認証失敗
            $authFailed = true;

            return $this->view->render(
                $response,
                'admin/views/auth/login.twig',
                compact('form', 'authFailed')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     */
    public function logout(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $auth = $this->c->get('auth');

        // ログを残すために、セッションを削除するまえに一旦ログインユーザーを取得する
        $auth->authenticated();
        // ログアウトする
        $auth->logout();
        $this->flash->addMessage('success', 'ログアウトしました。');

        return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.auth.login'))->withStatus(302);
    }
}
