<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Controllers;

use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use Psr\Http\Message\ResponseInterface;
use Rakit\Validation\RuleQuashException;
use MicroEngine\Admin\Forms\BookmarkForm;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;

class BookmarkController extends Controller
{
    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $user = $this->c->get('auth')->getUser();
        $bookmarks = $user->getBookmarks();

        return $this->view->render(
            $response,
            'admin/views/bookmark/index.twig',
            compact('bookmarks')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function edit(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new BookmarkForm();
        $user = $this->c->get('auth')->getUser();

        $wrappedData = [];
        foreach ($user->getBookmarks() as $row) {
            $wrappedData[] = array_values($row);
        }

        if ($request->getMethod() === 'POST') {
            // POSTで遷移した場合のみ、パラメータを受け付けて、最後のデータとして追加する
            $body = $request->getParsedBody();
            $name = $body['name'] ?? null;
            $url = $body['url'] ?? null;
            if (!empty($url)) {
                $wrappedData[] = [$name, $url];
            }
        }

        $json = json_encode($wrappedData);

        $form->setInputValue([
            'data' => $json,
        ]);

        return $this->view->render(
            $response,
            'admin/views/bookmark/edit.twig',
            compact('form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuleQuashException
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function update(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new BookmarkForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $data = $form->getInputValues();
            $id = $this->c->get('auth')->getUserId();

            // テーブルサービスを使ってデータを調整する
            $tableManager = $this->c->get('tableManager');
            $data = $tableManager->unwrapData($data, [
                'data_type' => 'matrix',
                'columns' => [
                    'name' => '名前',
                    'url' => 'URL',
                ]
            ]);
            $data = [
                'bookmarks' => $data['data']
            ];

            $this->c->get('userManager')->update($id, $data);
            $this->flash->addMessage('success', 'ブックマークを更新しました。');

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.bookmark.index'))->withStatus(302);
        } else {
            return $this->view->render(
                $response,
                'admin/views/bookmark/edit.twig',
                compact('form')
            );
        }
    }
}
