<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Controllers;

use Slim\App;
use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use Psr\Http\Message\ResponseInterface;
use MicroEngine\Admin\Forms\LicenseForm;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;

class LicenseController extends Controller
{
    /**
     * LicenseController constructor.
     * @param App $app
     */
    public function __construct(App $app)
    {
        parent::__construct($app);

        $this->c->get('userManager')->required('license');
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        // ライセンス情報取得
        $info = $this->c->get('license')->info();

        return $this->view->render(
            $response,
            'admin/views/license/index.twig',
            compact('info')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function register(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new LicenseForm($this->c);

        return $this->view->render(
            $response,
            'admin/views/license/register.twig',
            compact('form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function store(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new LicenseForm($this->c);
        $form->setInputValue($request->getUploadedFiles());

        // バリデーション
        if ($form->validate()) {
            // ライセンスファイルを保存する
            $this->c->get('license')->store($form->get('license'));
            $this->flash->addMessage('success', 'ライセンスを登録しました。');

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.license.index'))->withStatus(302);
        } else {
            return $this->view->render(
                $response,
                'admin/views/license/register.twig',
                compact('form')
            );
        }
    }
}
