<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Controllers;

use Slim\App;
use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use Psr\Http\Message\ResponseInterface;
use MicroEngine\Core\Support\SystemInfo;
use Rakit\Validation\RuleQuashException;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;

class SystemController extends Controller
{
    /**
     * SystemController constructor.
     * @param App $app
     */
    public function __construct(App $app)
    {
        parent::__construct($app);

        $this->c->get('userManager')->required('system');
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $systemInfo = new SystemInfo($this->c);
        $software = $systemInfo->software();
        $server = $systemInfo->server();
        $database = $systemInfo->database();
        $php = $systemInfo->php();
        $client = $systemInfo->client();

        return $this->view->render(
            $response,
            'admin/views/system/index.twig',
            compact('software', 'server', 'database', 'php', 'client')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     */
    public function phpinfo(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $systemInfo = new SystemInfo($this->c);
        $phpinfo = $systemInfo->phpinfo();
        $response->getBody()->write($phpinfo);

        return $response;
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function libs(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $systemInfo = new SystemInfo($this->c);
        $libs = $systemInfo->libs();

        return $this->view->render(
            $response,
            'admin/views/system/libs.twig',
            compact('libs')
        );
    }
}
