<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms;

use Slim\App;
use MicroEngine\Admin\Middleware\AuthMiddleware;
use Slim\Interfaces\RouteCollectorProxyInterface;
use MicroEngine\Core\Middleware\DBTransactionMiddleware;
use MicroEngine\Install\Middleware\SchemaVersionMiddleware;

class CmsRoute
{
    /**
     * ルート登録
     * @param App $app
     */
    public function register(App $app)
    {
        $c = $app->getContainer();

        $app->group('/admin', function (RouteCollectorProxyInterface $group) {
            // カテゴリ
            $group->group('/category', function (RouteCollectorProxyInterface $group) {
                $group->get('', 'categoryController:index')->setName('admin.category.index');
                $group->get('/create', 'categoryController:create')->setName('admin.category.create');
                $group->post('', 'categoryController:store')->setName('admin.category.store');
                $group->get('/{id:[0-9]+}[/page/{page_no:[0-9]+}/{sort}/{order}]', 'categoryController:show')->setName('admin.category.show');
                $group->get('/{id:[0-9]+}/edit', 'categoryController:edit')->setName('admin.category.edit');
                $group->put('/{id:[0-9]+}', 'categoryController:update')->setName('admin.category.update');
                $group->delete('/{id:[0-9]+}', 'categoryController:destroy')->setName('admin.category.destroy');
                $group->get('/modal', 'categoryController:modal')->setName('admin.category.modal');
                $group->get('/modal/{id:[0-9]+}', 'categoryController:modalShow')->setName('admin.category.modalShow');
            });
            // フィールド
            $group->group('/field', function (RouteCollectorProxyInterface $group) {
                $group->get('/field/{name}/{parentField}/{index}', 'fieldController:field')->setName('admin.field.field');
            });
            // ページ
            $group->group('/page', function (RouteCollectorProxyInterface $group) {
                $group->get('', 'pageController:index')->setName('admin.page.index');
                $group->get('/content/{type}[/page/{page_no:[0-9]+}/{sort}/{order}]', 'pageController:listing')->setName('admin.page.listing');
                $group->get('/content/{type}/create', 'pageController:create')->setName('admin.page.create');
                $group->get('/content/{id:[0-9]+}/copy', 'pageController:copy')->setName('admin.page.copy');
                $group->post('/content/{type}', 'pageController:store')->setName('admin.page.store');
                $group->map(['POST', 'PUT'], '/content/{type}/preview', 'pageController:preview')->setName('admin.page.preview');
                $group->get('/content/{id:[0-9]+}/edit[/{edit_mode}/{revision_id:[0-9]+}]', 'pageController:edit')->setName('admin.page.edit');
                $group->put('/content/{id:[0-9]+}', 'pageController:update')->setName('admin.page.update');
                $group->get('/content/{id:[0-9]+}/info', 'pageController:info')->setName('admin.page.info');
                $group->get('/content/{id:[0-9]+}/view[/{revision_id:[0-9]+}]', 'pageController:view')->setName('admin.page.view');
                $group->delete('/content/{id:[0-9]+}[/{edit_mode}/{revision_id:[0-9]+}]', 'pageController:destroy')->setName('admin.page.destroy');
                $group->get('/trash', 'pageController:trash')->setName('admin.page.trash');
                $group->delete('/trash/{id:[0-9]+}', 'pageController:forceDestroy')->setName('admin.page.forceDestroy');
                $group->delete('/trash', 'pageController:emptyTrash')->setName('admin.page.emptyTrash');
            });
            // コレクション
            $group->group('/collection', function (RouteCollectorProxyInterface $group) {
                $group->get('', 'collectionController:index')->setName('admin.collection.index');
                $group->get('/content/{type}[/page/{page_no:[0-9]+}/{sort}/{order}]', 'collectionController:listing')->setName('admin.collection.listing');
                $group->get('/content/{type}/create', 'collectionController:create')->setName('admin.collection.create');
                $group->get('/content/{id:[0-9]+}/copy', 'collectionController:copy')->setName('admin.collection.copy');
                $group->post('/content/{type}', 'collectionController:store')->setName('admin.collection.store');
                $group->map(['POST', 'PUT'], '/content/{type}/preview', 'collectionController:preview')->setName('admin.collection.preview');
                $group->get('/content/{id:[0-9]+}/edit[/{edit_mode}/{revision_id:[0-9]+}]', 'collectionController:edit')->setName('admin.collection.edit');
                $group->put('/content/{id:[0-9]+}', 'collectionController:update')->setName('admin.collection.update');
                $group->get('/content/{id:[0-9]+}/info', 'collectionController:info')->setName('admin.collection.info');
                $group->delete('/content/{id:[0-9]+}[/{edit_mode}/{revision_id:[0-9]+}]', 'collectionController:destroy')->setName('admin.collection.destroy');
                $group->get('/trash', 'collectionController:trash')->setName('admin.collection.trash');
                $group->delete('/trash/{id:[0-9]+}', 'collectionController:forceDestroy')->setName('admin.collection.forceDestroy');
                $group->delete('/trash', 'collectionController:emptyTrash')->setName('admin.collection.emptyTrash');
            });
            // モジュール
            $group->group('/module', function (RouteCollectorProxyInterface $group) {
                $group->get('', 'moduleController:index')->setName('admin.module.index');
                $group->get('/content/{type}/create', 'moduleController:create')->setName('admin.module.create');
                $group->post('/content/{type}', 'moduleController:store')->setName('admin.module.store');
                $group->map(['POST', 'PUT'], '/content/{type}/preview', 'moduleController:preview')->setName('admin.module.preview');
                $group->get('/content/{id:[0-9]+}/edit[/{edit_mode}/{revision_id:[0-9]+}]', 'moduleController:edit')->setName('admin.module.edit');
                $group->put('/content/{id:[0-9]+}', 'moduleController:update')->setName('admin.module.update');
                $group->get('/content/{id:[0-9]+}/info', 'moduleController:info')->setName('admin.module.info');
                $group->delete('/content/{id:[0-9]+}[/{revision_id:[0-9]+}]', 'moduleController:destroy')->setName('admin.module.destroy');
            });
            // カレンダー
            $group->group('/calendar', function (RouteCollectorProxyInterface $group) {
                $group->get('', 'calendarController:index')->setName('admin.calendar.index');
                $group->get('/content/{type}[/{date:[-0-9]+}]', 'calendarController:listing')->setName('admin.calendar.listing');
                $group->get('/content/{type}/create/{date:[-0-9]+}', 'calendarController:create')->setName('admin.calendar.create');
                $group->get('/content/{id:[0-9]+}/copy', 'calendarController:copy')->setName('admin.calendar.copy');
                $group->post('/content/{type}', 'calendarController:store')->setName('admin.calendar.store');
                $group->map(['POST', 'PUT'], '/content/{type}/preview', 'calendarController:preview')->setName('admin.calendar.preview');
                $group->get('/content/{id:[0-9]+}/edit[/{edit_mode}/{revision_id:[0-9]+}]', 'calendarController:edit')->setName('admin.calendar.edit');
                $group->put('/content/{id:[0-9]+}', 'calendarController:update')->setName('admin.calendar.update');
                $group->delete('/content/{id:[0-9]+}[/{edit_mode}/{revision_id:[0-9]+}]', 'calendarController:destroy')->setName('admin.calendar.destroy');
                $group->get('/content/{id:[0-9]+}/bulk/copy', 'calendarController:bulkCopy')->setName('admin.calendar.bulkCopy');
                $group->post('/content/{id:[0-9]+}/bulk', 'calendarController:bulkStore')->setName('admin.calendar.bulkStore');
                $group->get('/content/{id:[0-9]+}/bulk/takeDown', 'calendarController:bulkTakeDown')->setName('admin.calendar.bulkTakeDown');
                $group->delete('/content/{id:[0-9]+}/bulk', 'calendarController:bulkDestroy')->setName('admin.calendar.bulkDestroy');
                $group->get('/content/{id:[0-9]+}/info', 'calendarController:info')->setName('admin.calendar.info');
                $group->get('/content/{type}/{date:[-0-9]+}/yearly', 'calendarController:yearly')->setName('admin.calendar.yearly');
                $group->get('/trash', 'calendarController:trash')->setName('admin.calendar.trash');
                $group->delete('/trash/{id:[0-9]+}', 'calendarController:forceDestroy')->setName('admin.calendar.forceDestroy');
                $group->delete('/trash', 'calendarController:emptyTrash')->setName('admin.calendar.emptyTrash');
            });
            // テーブル
            $group->group('/table', function (RouteCollectorProxyInterface $group) {
                $group->get('', 'tableController:index')->setName('admin.table.index');
                $group->get('/{type}/create', 'tableController:create')->setName('admin.table.create');
                $group->post('/{type}', 'tableController:store')->setName('admin.table.store');
                $group->get('/{id:[0-9]+}/edit[/{edit_mode}/{revision_id:[0-9]+}]', 'tableController:edit')->setName('admin.table.edit');
                $group->put('/{id:[0-9]+}', 'tableController:update')->setName('admin.table.update');
                $group->get('/{id:[0-9]+}/info', 'tableController:info')->setName('admin.table.info');
                $group->delete('/{id:[0-9]+}[/{revision_id:[0-9]+}]', 'tableController:destroy')->setName('admin.table.destroy');
            });
            // ファイル
            $group->group('/file', function (RouteCollectorProxyInterface $group) {
                $group->get('/manager[/{sort}/{order}]', 'fileController:manager')->setName('admin.file.manager');
                $group->post('/uploads', 'fileController:uploads')->setName('admin.file.uploads');
                $group->get('/{id:[0-9]+}/edit', 'fileController:edit')->setName('admin.file.edit');
                $group->put('/{id:[0-9]+}', 'fileController:update')->setName('admin.file.update');
                $group->get('/{id:[0-9]+}/info', 'fileController:info')->setName('admin.file.info');
                $group->delete('/{id:[0-9]+}', 'fileController:destroy')->setName('admin.file.destroy');
                // ディレクトリ
                $group->get('/dir/create', 'fileController:createDir')->setName('admin.file.createDir');
                $group->post('/dir', 'fileController:storeDir')->setName('admin.file.storeDir');
                $group->get('/dir/{id:[0-9]+}/edit', 'fileController:editDir')->setName('admin.file.editDir');
                $group->put('/dir/{id:[0-9]+}', 'fileController:updateDir')->setName('admin.file.updateDir');
                $group->delete('/dir/{id:[0-9]+}', 'fileController:destroyDir')->setName('admin.file.destroyDir');
                // モーダル
                $group->get('/modal/dir/create', 'fileController:modalCreateDir')->setName('admin.file.modal.createDir');
                $group->get('/modal/{sort}/{order}', 'fileController:modal')->setName('admin.file.modal');
                $group->post('/modal/dir', 'fileController:modalStoreDir')->setName('admin.file.modal.storeDir');
                // ファイル同期
                $group->post('/sync', 'fileController:sync')->setName('admin.file.sync');
            });
            // テーマ
            $group->group('/theme', function (RouteCollectorProxyInterface $group) {
                $group->get('/manager', 'themeController:manager')->setName('admin.theme.manager');
                $group->post('/uploads', 'themeController:uploads')->setName('admin.theme.uploads');
                $group->post('/downloads', 'themeController:downloads')->setName('admin.theme.downloads');
                // ファイル
                $group->get('/file/edit', 'themeController:editFile')->setName('admin.theme.editFile');
                $group->put('/file', 'themeController:updateFile')->setName('admin.theme.updateFile');
                $group->delete('/file', 'themeController:destroy')->setName('admin.theme.destroy');
                // ディレクトリ
                $group->get('/dir/create', 'themeController:createDir')->setName('admin.theme.createDir');
                $group->post('/dir', 'themeController:storeDir')->setName('admin.theme.storeDir');
            });
            // バックアップ
            $group->group('/backup', function (RouteCollectorProxyInterface $group) {
                $group->get('', 'backupController:index')->setName('admin.backup.index');
                $group->get('/create', 'backupController:create')->setName('admin.backup.create');
                $group->post('', 'backupController:store')->setName('admin.backup.store');
                $group->get('/info/{filename}', 'backupController:info')->setName('admin.backup.info');
                $group->post('/restore/{filename}', 'backupController:restore')->setName('admin.backup.restore');
                $group->get('/download/{filename}', 'backupController:download')->setName('admin.backup.download');
                $group->delete('', 'backupController:destroy')->setName('admin.backup.destroy');
            });
            // キャッシュ
            $group->group('/cache', function (RouteCollectorProxyInterface $group) {
                $group->get('', 'cacheController:index')->setName('admin.cache.index');
                $group->post('', 'cacheController:clear')->setName('admin.cache.clear');
            });
        })->add(new AuthMiddleware($c))->add(new SchemaVersionMiddleware($c))->add(new DBTransactionMiddleware($c));
    }
}
