<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms;

use Psr\Container\ContainerInterface;
use MicroEngine\Cms\Views\CmsExtension;
use MicroEngine\Core\Support\ServiceProvider;
use MicroEngine\Cms\Services\{
    Backup, Data, CodeEditor, Import, ImageUtility, Theme, ContentService
};
use MicroEngine\Cms\Managers\{
    FileManager, PageManager, TableManager, CategoryManager, CalendarManager, CollectionManager, ModuleManager
};
use MicroEngine\Cms\Controllers\{
    BackupController,
    CacheController,
    PageController,
    FileController,
    FieldController,
    TableController,
    ThemeController,
    ModuleController,
    ContentController,
    CalendarController,
    CategoryController,
    CollectionController
};

/**
 * @package MicroEngine\Cms
 */
class CmsServiceProvider extends ServiceProvider
{
    /**
     * サービス登録
     */
    public function register()
    {
        $c = $this->c;

        $c->set('categoryManager', function (ContainerInterface $c) {
            return new CategoryManager($c);
        });
        $c->set('pageManager', function (ContainerInterface $c) {
            return new PageManager($c);
        });
        $c->set('collectionManager', function (ContainerInterface $c) {
            return new CollectionManager($c);
        });
        $c->set('moduleManager', function (ContainerInterface $c) {
            return new ModuleManager($c);
        });
        $c->set('calendarManager', function (ContainerInterface $c) {
            return new CalendarManager($c);
        });
        $c->set('contentService', function (ContainerInterface $c) {
            return new ContentService($c);
        });
        $c->set('fileManager', function (ContainerInterface $c) {
            return new FileManager($c);
        });
        $c->set('tableManager', function (ContainerInterface $c) {
            return new TableManager($c);
        });
        $c->set('theme', function (ContainerInterface $c) {
            return new Theme($c);
        });
        $c->set('backup', function (ContainerInterface $c) {
            return new Backup($c);
        });
        $c->set('import', function (ContainerInterface $c) {
            return new Import($c);
        });
        $c->set('data', function (ContainerInterface $c) {
            return new Data($c);
        });
        $c->set('imageUtility', function (ContainerInterface $c) {
            return new ImageUtility($c);
        });
        $c->set('codeEditor', function (ContainerInterface $c) {
            return new CodeEditor($c);
        });
        $c->set('categoryController', function () {
            return new CategoryController($this->app);
        });
        $c->set('pageController', function () {
            return new PageController($this->app);
        });
        $c->set('collectionController', function () {
            return new CollectionController($this->app);
        });
        $c->set('moduleController', function () {
            return new ModuleController($this->app);
        });
        $c->set('calendarController', function () {
            return new CalendarController($this->app);
        });
        $c->set('fieldController', function () {
            return new FieldController($this->app);
        });
        $c->set('tableController', function () {
            return new TableController($this->app);
        });
        $c->set('fileController', function () {
            return new FileController($this->app);
        });
        $c->set('themeController', function () {
            return new ThemeController($this->app);
        });
        $c->set('backupController', function () {
            return new BackupController($this->app);
        });
        $c->set('cacheController', function () {
            return new CacheController($this->app);
        });
    }

    /**
     * サービス起動
     */
    public function boot()
    {
        $c = $this->c;

        // Twigエクステンションを登録
        $c->get('view')->addExtension(new CmsExtension($c));

        // Register routes
        $route = new CmsRoute();
        $route->register($this->app);
    }
}
