<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Controllers;

use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use Psr\Http\Message\ResponseInterface;
use MicroEngine\Core\Controllers\Controller;
use Psr\Http\Message\ServerRequestInterface;

class CacheController extends Controller
{
    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $htmlCache = $this->c->get('htmlCache');
        // 有効期限取得
        $expirationDate = $htmlCache->expirationDate();
        // キャッシュ作成日時取得
        $createdAt = $htmlCache->createdAt();

        return $this->view->render(
            $response,
            'cms/views/cache/index.twig',
            compact('expirationDate', 'createdAt')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     */
    public function clear(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        // HTMLキャッシュ削除を実行する
        $this->c->get('htmlCache')->clear();
        $this->flash->addMessage('success', 'HTMLキャッシュを削除しました。');

        return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.cache.index'))->withStatus(302);
    }
}
