<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Services;

use Illuminate\Support\Arr;
use Psr\Container\ContainerInterface;

class CodeEditor
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var array
     */
    protected $config;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
        $this->config = config('cms.code_editor');
    }

    /**
     * 拡張子を元にコードエディターで編集可能かどうかを判定する
     *
     * @param string $path
     * @return bool
     */
    public function isEditable(string $path): bool
    {
        $extension = pathinfo($path, PATHINFO_EXTENSION);
        $extensions = Arr::flatten($this->config['modes']);

        return in_array($extension, $extensions);
    }

    /**
     * 拡張子からコードエディターの編集モードを判定する
     *
     * @param string $path
     * @return string $mode
     */
    public function getMode(string $path): string
    {
        $extension = pathinfo($path, PATHINFO_EXTENSION);
        $mode = 'text/plain';
        foreach ($this->config['modes'] as $name => $modeList) {
            if (in_array($extension, $modeList)) {
                $mode = $name;
                break;
            }
        }
        return $mode;
    }
}
