<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2020 MicroEngine Inc.
 */

namespace MicroEngine\Core\Handler;

use Slim\Handlers\ErrorHandler;
use MicroEngine\Core\Exception\MeException;

class CustomErrorHandler extends ErrorHandler
{
    protected function logError(string $error): void
    {
        // 詳細ログ設定が有効な場合は、PHPのerror_log()を使ってログを出力する
        if ($this->logErrorDetails) {
            parent::logError($error);
        }

        // このアプリケーション用のログを記録する
        global $app;
        $logger = $app->getContainer()->get('logger');

        $class = get_class($this->exception);
        $message = $this->exception->getMessage();
        if ($this->statusCode < 500) {
            // ステータスコード\が500未満はデバッグレベルで記録する
            $logger->debug($message);
        } else if ($class === MeException::class) {
            // MeExceptionはエラーレベルで記録する
            $logger->error($message);
        } else {
            // MeException以外の例外はワーニングレベルで記録する
            $logger->warning($message);
        }
    }
}
