<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Middleware;

use Slim\Psr7\Response;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Illuminate\Database\Capsule\Manager as DB;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;

/**
 * DBトランザクション管理ミドルウェア
 * エラーや例外発生時にトランザクションをロールバックして中途半端なDB更新を防ぐ。
 *
 * @package MicroEngine\Core\Middleware
 */
class DBTransactionMiddleware extends Middleware
{
    /**
     * @param ServerRequestInterface $request
     * @param RequestHandler $handler
     * @return Response
     * @throws \Exception
     */
    public function __invoke(Request $request, RequestHandler $handler): Response
    {
        // トランザクションを開始する
        DB::beginTransaction();

        try {
            // 本体処理を実行する
            $response = $handler->handle($request);
        } catch (\Exception $e) {
            // 例外が発生した場合はロールバックして、中途半端にコミットしてしまわないようにする
            DB::rollBack();
            throw $e;
        }

        if ($response instanceof ResponseInterface && $response->getStatusCode() >= 400) {
            // ステータスコードが400以上であればロールバックする
            DB::rollBack();
        } else {
            // ステータスコードが400未満であればコミットする
            DB::commit();
        }
        return $response;
    }
}
