<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Support;

class Base64Stream
{
    /**
     * @var string スキーム名
     */
    const SCHEME = 'base64';
    /**
     * @var int ファイル内の位置
     */
    protected $position;
    /**
     * @var array ファイル情報
     */
    protected $stat;
    /**
     * @var string 対象ファイルの内容
     */
    protected $data;
    /**
     * @var bool ストリームラッパーを登録済みかどうか
     */
    protected static $registered = false;

    /**
     * ストリームラッパーを登録する
     */
    public static function register()
    {
        if (self::$registered === false) {
            self::$registered = true;
            stream_wrapper_register(self::SCHEME, __CLASS__);
        }
    }

    /**
     * ストリームを開く
     *
     * @param string $path
     * @return bool
     */
    public function stream_open($path): bool
    {
        // scheme:// 以降のパスを取得
        $path = substr($path, strlen(self::SCHEME) + 3);

        $this->position = 0;
        $this->stat = stat($path);
        $this->data = file_get_contents($path);

        if ($this->data === false) {
            return false;
        }
        $this->data = $this->decode($this->data);

        return true;
    }

    /**
     * ストリームを読み込む
     *
     * @param int $count
     * @return bool|string
     */
    public function stream_read($count)
    {
        $ret = substr($this->data, $this->position, $count);
        $this->position += strlen($ret);
        return $ret;
    }

    /**
     * ストリームの最後まで到達したかどうか
     *
     * @return bool
     */
    public function stream_eof(): bool
    {
        return $this->position >= strlen($this->data);
    }

    /**
     * ファイル情報を返す
     *
     * @return array
     */
    public function stream_stat()
    {
        return $this->stat;
    }

    /**
     * オプション設定
     *
     * @param int $option
     * @param int $arg1
     * @param int $arg2
     * @return bool
     */
    public function stream_set_option(int $option, int $arg1, int $arg2): bool
    {
        return false;
    }

    /**
     * ファイルからbase64部分を取り出して、decodeする
     *
     * @param string $data
     * @return bool|string
     */
    protected function decode(string $data)
    {
        $offset = strpos($data, '?>') + 2;
        $encoded = substr($data, $offset);

        return base64_decode($encoded);
    }
}
