<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Install\Forms;

use Rakit\Validation\ErrorBag;
use MicroEngine\Core\Forms\Form;
use Illuminate\Database\Capsule\Manager;
use Psr\Container\ContainerInterface;

class DatabaseSetupForm extends Form
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var array
     */
    protected $inputs = [
        // 接続エラーを表示するためのアイテム。HTMLのフォーム部品は持たない。
        'connection' => [
            'label' => 'データベース接続',
            'value' => '',
            'rule' => '',
        ],
        'host' => [
            'label' => 'ホスト名',
            'value' => '127.0.0.1',
            'rule' => 'required',
        ],
        'port' => [
            'label' => 'ポート番号',
            'value' => '3306',
            'rule' => 'numeric',
        ],
        'database' => [
            'label' => 'データベース名',
            'value' => 'mecms',
            'rule' => 'required',
        ],
        'username' => [
            'label' => 'ユーザー名',
            'value' => 'user',
            'rule' => 'required',
        ],
        'password' => [
            'label' => 'パスワード',
            'value' => '',
            'rule' => '',
        ],
    ];

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
    }

    /**
     * データベース接続テスト
     * @return bool
     */
    public function validateConnection(): bool
    {
        // フォームの入力値を取得
        $config = $this->getInputValues();
        // ドライバー設定などを現在の設定ファイルから取得してセットする
        $connection = config('db.default', 'mysql');
        $config += config('db.connections.' . $connection);

        // データベースマネージャに設定を登録する
        $manager = new Manager();
        $manager->addConnection($config);

        try {
            // PDOオブジェクトを取得できるかどうかでコネクションが成立したかどうかを確認する
            $pdo = $manager->getConnection()->getPdo();
        } catch (\Exception $e) {
            $errorMessage = $e->getMessage();
        }

        if (isset($errorMessage)) {
            $message = "データベースとの接続に失敗しました。データベース設定を見直してください。\nエラーメッセージ：" . $errorMessage;
            $this->errorBag = new ErrorBag();
            $this->errorBag->add(
                'database_connection',
                'connection',
                $message
            );
            return false;
        }

        $count = 0;
        try {
            $sql = 'SELECT count(*) from users';
            $stmt = $pdo->prepare($sql);
            $stmt->execute();
            $count = $stmt->fetchColumn();
        } catch (\Exception $e) {
            // インストール前はテーブルが存在しないので例外が発生する
        }

        // インストール済みのチェック
        if ($count > 0) {
            $this->errorBag = new ErrorBag();
            $this->errorBag->add(
                'database_connection',
                'installed',
                'このデータベースにはすでにテーブルが存在するため、インストールを実行することができません。'
            );
            return false;
        }

        // DB接続、インストール済みのエラーがなければ errorBagを空にしてtrueを返す
        $this->errorBag = null;

        return true;
    }
}
