<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Site\Controllers;

use Slim\Routing\RouteContext;
use MicroEngine\Cms\Models\Page;
use MicroEngine\Cms\Services\Query;
use MicroEngine\Site\Forms\SearchForm;
use Psr\Http\Message\ResponseInterface;
use MicroEngine\Core\Controllers\Action;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Exception\CustomNotFoundException;

class SearchAction extends Action
{
    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @param Page|null $page
     * @return ResponseInterface
     * @throws CustomNotFoundException
     */
    public function __invoke(ServerRequestInterface $request, ResponseInterface $response, array $args, Page $page = null): ResponseInterface
    {
        $pageNo = $args['page_no'] ?? 1;

        // ルート名・タイプを取得
        $route = RouteContext::fromRequest($request)->getRoute();
        $routeName = $route->getName();
        if ($routeName !== null) {
            $type = $routeName;
        } else {
            // RoutingActionから転送された場合
            $type = $page->type;
            $routeName = $type;
        }
        // 設定を取得
        $config = config('pages.' . $type);

        // 検索フォーム作成
        $form = new SearchForm();
        $inputs = $config['form']['inputs'] ?? [];
        // 入力設定をセット
        $form->setInputs($inputs);
        // get/post値をセットする
        $queryParams = $request->getQueryParams() ?? [];
        $postData = $request->getParsedBody() ?? [];
        $form->setInputValue(array_merge($queryParams, $postData));

        // 該当コンテンツのオプション設定を取得
        $options = $config['options'];

        // フォームからパラメータを取り出す。
        $params = $form->getParams();

        // クエリ作成
        $query = new Query($this->c);
        $pagination = $query->pagination($config['options']['query'], $pageNo, $options, $params);

        // ルート情報をセット
        $pagination->setRouteInfo($this->c, $routeName, [], $params);

        // ページ範囲を超えていないか確認
        if ($pagination->outOfBounds()) {
            throw new CustomNotFoundException($request);
        }

        // 何ページ目を表示する場合でも、１ページ目の情報をDBから取得する
        if (!isset($page)) {
            $page = Page::where('type', $type)->first();
            if (!$page) {
                throw new CustomNotFoundException($request);
            }
        }
        // １ページ目のパスをセット
        $pagination->setFirstPath($page->path);
        $this->c->get('site')->pushContent($page, true);

        return $this->c->get('view')->render(
            $response,
            $page->getTemplatePath(),
            compact('pagination', 'page', 'form')
        );
    }
}
