<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Site\Middleware;

use Slim\Psr7\Response;
use MicroEngine\Core\Middleware\Middleware;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;

class HtmlCacheMiddleware extends Middleware
{
    /**
     * @param ServerRequestInterface $request
     * @param RequestHandler $handler
     * @return Response
     */
    public function __invoke(Request $request, RequestHandler $handler): Response
    {
        $response = $handler->handle($request);

        // httpメソッドを確認
        if ($request->getMethod() !== 'GET') {
            // getメソッド以外はスルー
            return $response;
        }

        if ($this->c->get('auth')->authenticated()) {
            // ログイン中はキャッシュを作成しない。
            return $response;
        }

        $htmlCache = $this->c->get('htmlCache');
        if (!$htmlCache->isEnabled()) {
            // キャッシュ機能が無効の場合はスルー
            return $response;
        }

        // 有効期限確認
        $htmlCache->checkExpired();

        $path = $request->getUri()->getPath();
        if (!$htmlCache->has($path)) {
            // キャッシュファイルが無ければファイルを保存する。
            $body = $response->getBody();
            $body->rewind();
            $content = $body->getContents();
            $htmlCache->put($path, $content);
        }
        return $response;
    }
}
