<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Site;

use Slim\App;
use Slim\Interfaces\RouteCollectorProxyInterface;
use MicroEngine\Site\Middleware\EditLinkMiddleware;
use MicroEngine\Site\Middleware\HtmlCacheMiddleware;
use MicroEngine\Install\Middleware\SchemaVersionMiddleware;

class SiteRoute
{
    /**
     * ルート登録
     * @param App $app
     */
    public function register(App $app)
    {
        $c = $app->getContainer();

        // ファイルコンテンツ用ルート
        $app->get('/files/{path:.*}', Controllers\FilesAction::class);

        // 公開ページグループ
        $app->group('', function (RouteCollectorProxyInterface $group) use ($app) {
            $pages = config('pages');

            foreach ($pages as $type => $content) {
                if (!isset($content['action'])) {
                    continue;
                }
                // アクション用ルート
                if ($content['action'] === 'mapping') {
                    $options = $content['options'];
                    $app->any($options['route'], Controllers\MappingAction::class)->setName($type);
                } elseif ($content['action'] === 'pagination') {
                    $options = $content['options'];
                    $app->any($options['route'], Controllers\PaginationAction::class)->setName($type);
                } elseif ($content['action'] === 'search') {
                    $options = $content['options'];
                    $app->any($options['route'], Controllers\SearchAction::class)->setName($type);
                } elseif ($content['action'] === 'archive') {
                    $options = $content['options'];
                    $app->any($options['route'], Controllers\ArchiveAction::class)->setName($type);
                } elseif ($content['action'] === 'calendar') {
                    $options = $content['options'];
                    $app->any($options['route'], Controllers\CalendarAction::class)->setName($type);
                }
            }
            // デフォルトの公開ページ用ルート
            $app->any('{path:.*}', Controllers\RoutingAction::class);
        })->add(new SchemaVersionMiddleware($c))->add(new HtmlCacheMiddleware($c))->add(new EditLinkMiddleware($c));
    }
}
