<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Site;

use MicroEngine\Site\Services\Site;
use Psr\Container\ContainerInterface;
use MicroEngine\Site\Services\HtmlCache;
use MicroEngine\Site\Views\SiteExtension;
use MicroEngine\Core\Support\ServiceProvider;

class SiteServiceProvider extends ServiceProvider
{
    /**
     * サービス登録
     */
    public function register()
    {
        $c = $this->c;

        $c->set('site', function(ContainerInterface $c){
            return new Site($c);
        });
        $c->set('htmlCache', function(ContainerInterface $c){
            return new HtmlCache($c);
        });
    }

    /**
     * サービス起動
     */
    public function boot()
    {
        $c = $this->c;

        $view = $c->get('view');
        $view->addExtension(new SiteExtension($c));
        if (file_exists(theme_root('views'))) {
            // テーマ名の指定ミスで管理画面までエラーになることを防ぐために、viewsディレクトリがあることを確認する
            $loader = $view->getLoader();
            $loader->addPath(theme_root('views'), 'site');
        }

        // Register routes
        $route = new SiteRoute();
        $route->register($this->app);
    }
}
