<?php

return [
    /**
     * 製品名
     */
    'name' => 'MicroEngine CMS',
    /**
     * バージョン番号
     */
    'version' => '2.2.0',
    /**
     * スキーマバージョン
     */
    'schema_version' => 1,
    /**
     * ソフトウェアのコピーライト年表記
     */
    'copyright_year' => '2018-2022',
    /**
     * 404ページのテンプレート
     */
    'template_404' => '@site/page/404.twig',
    /**
     * ナビゲーション
     */
    'navigation' => [
        'home' => [
            'name' => 'ホーム',
            'route' => 'admin.home.index',
            'menus' => [
                'dashboard' => [
                    'name' => 'ダッシュボード',
                    'route' => 'admin.home.index',
                    'icon' => 'fa fa-dashboard',
                ],
                'category' => [
                    'name' => 'サイト',
                    'route' => 'admin.category.index',
                    'icon' => 'fa fa-sitemap',
                ],
                'page' => [
                    'name' => 'ページ',
                    'route' => 'admin.page.index',
                    'icon' => 'fa fa-files-o',
                ],
                'collection' => [
                    'name' => 'コレクション',
                    'route' => 'admin.collection.index',
                    'icon' => 'fa fa-database',
                ],
                'module' => [
                    'name' => 'モジュール',
                    'route' => 'admin.module.index',
                    'icon' => 'fa fa-cube',
                ],
                'calendar' => [
                    'name' => 'カレンダー',
                    'route' => 'admin.calendar.index',
                    'icon' => 'fa fa-calendar',
                ],
                'table' => [
                    'name' => 'テーブル',
                    'route' => 'admin.table.index',
                    'icon' => 'fa fa-table',
                ],
                'file' => [
                    'name' => 'ファイル',
                    'route' => 'admin.file.manager',
                    'icon' => 'fa fa-file',
                ],
                'theme' => [
                    'name' => 'テーマ',
                    'route' => 'admin.theme.manager',
                    'icon' => 'fa fa-newspaper-o',
                ],
            ],
        ],
        'system' => [
            'name' => 'システム',
            'route' => 'admin.user.index',
            'display_order' => 100,
            'menus' => [
                'user' => [
                    'name' => 'ユーザー',
                    'route' => 'admin.user.index',
                    'icon' => 'fa fa-user',
                ],
                'bookmark' => [
                    'name' => 'ブックマーク',
                    'route' => 'admin.bookmark.index',
                    'icon' => 'fa fa-star-o',
                ],
                'cache' => [
                    'name' => 'キャッシュ',
                    'route' => 'admin.cache.index',
                    'icon' => 'fa fa-hdd-o',
                ],
                'backup' => [
                    'name' => 'バックアップ',
                    'route' => 'admin.backup.index',
                    'icon' => 'fa fa-archive',
                ],
                'log' => [
                    'name' => 'ログ',
                    'route' => 'admin.log.viewer',
                    'icon' => 'fa fa-database',
                ],
                'config' => [
                    'name' => '設定',
                    'route' => 'admin.config.index',
                    'icon' => 'fa fa-cogs',
                ],
                'system' => [
                    'name' => 'システム情報',
                    'route' => 'admin.system.index',
                    'icon' => 'fa fa-server',
                ],
                'license' => [
                    'name' => 'ライセンス',
                    'route' => 'admin.license.index',
                    'icon' => 'fa fa-drivers-license',
                ],
            ],
        ],
    ],
    /**
     * ロール
     */
    'roles' => [
        /**
         * システム管理者ロール
         */
        'administrator' => [
            'label' => 'システム管理者',
            'capability' => [
                'theme' => true,
                'user' => true,
                'user_own' => true,
                'config' => true,
                'log' => true,
                'license' => true,
                'system' => true,
                'backup' => true,
            ],
            'hidden_menus' => [
            ],
        ],
        /**
         * デザイナーロール
         */
        'designer' => [
            'label' => 'デザイナー',
            'capability' => [
                'theme' => true,
                'user' => false,
                'user_own' => true,
                'config' => true,
                'log' => true,
                'license' => false,
                'system' => false,
                'backup' => false,
            ],
            'hidden_menus' => [
                'system' => [
                    'license',
                    'system',
                    'backup',
                ],
            ],
        ],
        /**
         * 編集者ロール
         */
        'editor' => [
            'label' => '編集者',
            'capability' => [
                'theme' => false,
                'user' => false,
                'user_own' => true,
                'config' => false,
                'log' => false,
                'license' => false,
                'system' => false,
                'backup' => false,
            ],
            'hidden_menus' => [
                'home' => [
                    'theme',
                ],
                'system' => [
                    'config',
                    'log',
                    'license',
                    'system',
                    'backup',
                ],
            ],
        ],
    ],
    /**
     * 設定ファイル
     * 管理画面で編集できる設定ファイルを定義します。
     */
    'config_files' => [
        'cms.php' => 'CMS設定',
        'pages.php' => 'ページ設定',
        'collections.php' => 'コレクション設定',
        'modules.php' => 'モジュール設定',
        'calendars.php' => 'カレンダー設定',
        'fields.php' => 'フィールド設定',
        'tables.php' => 'テーブル設定',
        'image.php' => '画像設定',
        'query.php' => 'クエリ設定',
        'admin.php' => '管理画面設定',
        'settings.php' => '基本設定',
    ],

    /**
     * サービスプロバイダ登録
     */
    'services' => [
        'Core',
        'Admin',
        'Cms',
        'Install',
        'Site',
    ],
    /**
     * ミドルウェアでルートを扱うための設定
     */
    'determineRouteBeforeAppMiddleware' => true,
];
