/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

(function ($, window) {
  'use strict';

  if (typeof window.mecms === 'undefined') {
    window.mecms = {};
  }

  /**
   * Code Editor
   */
  mecms.codeEditor = {
    codeMirror: null,
    query: null,
    init: function () {
      var self = this;

      $('button.search_btn').on('click', function (event) {
        self.search(this, event);
        return false;
      });
      $('#query').on('keydown', function (event) {
        return self.queryKeydown(this, event);
      });
    },
    startCodeMirror: function (requireList, mode, value) {
      var self = this, $codeEditor;

      require.config({
        packages: [{
          name: 'codemirror',
          location: '/' + mecms.themesPrefix + '/admin/vendor/codemirror/',
          main: 'lib/codemirror'
        }]
      });
      require(requireList, function (CodeMirror) {
        $codeEditor = $('#codeEditor').empty();
        mecms.codeEditor.codeMirror = CodeMirror($codeEditor.get(0), {
          mode: mode,
          value: value,
          autofocus: true,
          extraKeys: {'Alt-F': 'findPersistent'},
          lineNumbers: true,
          indentUnit: 4
        });
      });
    },
    search: function (thisObj, event) {
      var self = this;
      var query, presetQuery;
      var $this = $(thisObj);
      var prev = ($this.data('search') == 'prev');

      presetQuery = $this.data('query');
      if (presetQuery) {
        query = presetQuery;
        $('#query').val(query);
      } else {
        query = $('#query').val();
      }
      if (query != self.query) {
        self.query = query;
        self.codeMirror.setQuery(self.codeMirror, query);
      }
      if (event.shiftKey || prev) {
        self.codeMirror.execCommand('findPrev');
        $('#searchPrev').focus();
      } else {
        self.codeMirror.execCommand('findNext');
        $('#searchNext').focus();
      }
    },
    queryKeydown: function (thisObj, event) {
      var self = this;

      if (event.keyCode == 13) {
        self.search(thisObj, event);
        return false;
      }
      return true;
    },
    clearSearch: function () {
      var self = this;

      self.codeMirror.execCommand('clearSearch');
      $('#query').val('').focus();
      return false;
    }
  };

})(jQuery, window);
