/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

(function ($, window) {
  'use strict';

  if (typeof window.mecms === 'undefined') {
    window.mecms = {};
  }

  /**
   * Content
   */
  mecms.content = {
    baseType: null,
    type: null,
    filesPrefix: 'files',
    selectMode: 'page',
    selectHref: null,
    selectSrc: null,
    selectText: null,
    selectFilesize: null,
    codeSelector: null,
    attrId: null,
    attrClass: null,
    attrStyle: null,
    containersLength: {},
    $block: null,
    $iframe: null,
    updated: false,
    focusId: null,
    focusTop: 200,
    headerHeight: 50,
    repeatNo: null,
    dir: null,
    init: function (baseType, type, filesPrefix, open) {
      var self = this, $target;

      self.baseType = baseType;
      self.type = type;
      self.filesPrefix = filesPrefix;

      $(document).on('click', function (e) {
        $target = $(e.target);
        if (e.altKey) {
          // input部品の上でclickするとフォーカスがはずれないので、ボタンにフォーカスを移す
          $('#openPreviewPanel').focus().click();
        } else if ($target.is('[data-select-href], [data-select-src]')) {
          self.openModal(e);
        } else if ($target.parents('.editor__header--right, .screen-switcher').length === 0) {
          self.closeScreenSwitcher();
        }
      });
      $(document).on('click', '[data-code]', function (e) {
        self.openCodeEditor(e);
      });
      $('#formPanelBody').on('click', '[data-container] > *, [data-repeat-container] > *', function (e) {
        if ($(e.target).is('input, textarea, select, [data-meeditor-instance]')) {
          return true;
        }
        self.selectBlock(e);
      });
      $('#formPanelBody').on('click', '[name="popupFieldBox"]', function (e) {
        self.openAddFieldBox(e);
      });
      $('#formPanelBody').on('click', '[data-add-item]', function (e) {
        self.addItem(e);
      });
      $('#formPanelBody').on('click', '[data-attr-id]', function (e) {
        self.openAttrSettings(e);
      });
      $('#formPanelBody').on('click', '[data-img]', function (e) {
        self.focusId = $(e.target).attr('data-img');
        self.openPreviewPanel();
      });
      $('#formPanelBody').on('focus', '[data-container] > *, [data-repeat-container] > *', function (e) {
        self.selectBlock(e);
      });
      $('#formPanelBody').on('change', '[data-img-src]', function (e) {
        self.updateSrc(e);
      });
      $('#formPanelBody').on('change', 'input, select, textarea', function (e) {
        self.updated = true;
      });
      $('#formPanelBody').on('scroll', function (e) {
        self.openToolbox();
      });
      $('#contentPanelCol').on('mousedown', function (e) {
        self.findFocus();
      });
      $('#contentPanelCol').on('click', function (e) {
        if (!$(e.target).is('[data-container] *, [data-repeat-container] *')) {
          self.deselectBlock(e);
        }
      });
      $('#containerToolbox').on('keydown', function (e) {
        self.keyDown(e);
      });
      $('#addField').on('click', function (e) {
        self.addField(e);
      });
      $('#upField').on('click', function (e) {
        self.up(e);
      });
      $('#downField').on('click', function (e) {
        self.down(e);
      });
      $('#removeField').on('click', function (e) {
        self.remove(e);
      });
      self.countContainersLength();
      $('.add-field-btn, .add-field-box').on('click', '[data-field-name]', function (e) {
        self.getField(e);
      });
      $('#contentForm').bind('submit', function (e) {
        self.submitForm(e);
      });
      $('#codeModal').on('hidden.bs.modal', function (e) {
        self.closeCodeEditor(e);
      });
      $('#attrModal').on('hidden.bs.modal', function (e) {
        self.closeAttrSettings(e);
      });
      $('#formPanelHeader').on('click', '[data-width]', function (e) {
        self.screen(e);
      });
      $('#rotateScreen').on('click', function (e) {
        self.rotateScreen(e);
      });
      $('#openPreviewPanel').on('click', function (e) {
        self.openPreviewPanel();
      });
      $('#openScreenSwitcher').on('click', function (e) {
        $(this).toggleClass('btn-preview');
      });
      // openパラメータにpreview以外が指定されている場合は、プレビューパネルを閉じる
      if (open !== 'preview') {
        self.closePreviewPanel();
      }
      self.preview();
    },
    openPreviewPanel: function () {
      var self = this, $formPanelBody;

      $formPanelBody = $('#formPanelBody');
      if ($formPanelBody.is(':hidden')) {
        self.closePreviewPanel();
        return;
      }
      if (self.updated) {
        self.preview();
      }
      self.scrollPreview();
      $('#openPreviewPanel').addClass('btn-preview');
      $formPanelBody.fadeOut(100);
    },
    closePreviewPanel: function () {
      $('#formPanelBody').show();
      $('#openPreviewPanel').removeClass('btn-preview');
    },
    closeScreenSwitcher: function () {
      var $openScreenSwitcher;

      $openScreenSwitcher = $('#openScreenSwitcher');
      if (!$openScreenSwitcher.is('.collapsed')) {
        $openScreenSwitcher.click();
      }
    },
    openCodeEditor: function (e) {
      var self = this, $button, mode, $textarea, value;
      $button = $(e.currentTarget);

      self.codeSelector = $button.attr('data-code');
      if (!self.codeSelector.match(/^[\[.#]/)) {
        self.codeSelector = '#' + self.codeSelector;
      }

      mode = $button.attr('data-code-mode') || 'htmlmixed';
      $textarea = $(self.codeSelector);
      if ($textarea.is('[data-meeditor-instance]')) {
        value = $textarea.html();
      } else {
        value = $textarea.val();
      }

      $('#codeModal').modal('show');

      mecms.codeEditor.startCodeMirror([
        'codemirror',
        'codemirror/mode/htmlmixed/htmlmixed',
        'codemirror/mode/twig/twig',
        'codemirror/mode/css/css',
        'codemirror/mode/xml/xml',
        'codemirror/mode/javascript/javascript',
        'codemirror/mode/php/php',
        'codemirror/addon/dialog/dialog',
        'codemirror/addon/search/search',
        'codemirror/addon/search/searchcursor',
        'codemirror/addon/search/jump-to-line'
      ], mode, value);
    },
    closeCodeEditor: function (e) {
      var self = this, $textarea;

      $textarea = $(self.codeSelector);
      if ($textarea.is('[data-meeditor-instance]')) {
        $textarea.html(mecms.codeEditor.codeMirror.getDoc().getValue());
        $textarea = $('[data-meeditor="' + $textarea.attr('data-meeditor-instance') + '"]');
      }
      $textarea.val(mecms.codeEditor.codeMirror.getDoc().getValue());

      self.updated = true;
    },
    openAttrSettings: function (e) {
      var self = this, $button;

      $button = $(e.currentTarget);
      self.attrId = $button.attr('data-attr-id');
      self.attrClass = $button.attr('data-attr-class');
      self.attrStyle = $button.attr('data-attr-style');

      $('#attrIdSetting').val($('#' + self.attrId).val());
      $('#attrClassSetting').val($('#' + self.attrClass).val());
      $('#attrStyleSetting').val($('#' + self.attrStyle).val());

      $('#attrModal').modal('show');
    },
    closeAttrSettings: function (e) {
      var self = this;

      $('#' + self.attrId).val($('#attrIdSetting').val());
      $('#' + self.attrClass).val($('#attrClassSetting').val());
      $('#' + self.attrStyle).val($('#attrStyleSetting').val());
      self.updated = true;
    },
    // 選択モーダル
    setSelectProperty: function (e) {
      var self = this, $target;

      $target = $(e.target);
      self.selectMode = $target.attr('data-select-mode') || 'page';
      self.selectHref = $target.attr('data-select-href');
      self.selectSrc = $target.attr('data-select-src');
      self.selectText = $target.attr('data-select-text');
      self.selectFilesize = $target.attr('data-select-filesize');
      mecms.fileManager.isModal = true;

      $('#selectModal').empty().modal('show');
    },
    openModal: function (e) {
      var self = this;

      self.setSelectProperty(e);
      if (self.selectMode === 'page') {
        self.openLinkModal();
      } else {
        self.openFileModal();
      }
    },
    openLinkModal: function () {
      var self = this;
      self.showSelectModal('/admin/category/modal');
    },
    openLinkList: function (id) {
      var self = this;
      self.showSelectModal('/admin/category/modal/' + id);
    },
    openFileModal: function (dir, sort, order) {
      var self = this;

      if (!dir) {
        dir = '/';
      }
      if (!sort) {
        sort = 'updated_at';
      }
      if (!order) {
        order = 'desc';
      }
      self.dir = dir;
      self.showSelectModal('/admin/file/modal/' + sort + '/' + order + '?dir=' + dir + '&mode=' + self.selectMode);
    },
    showSelectModal: function (url) {
      var self = this, $modal;

      $modal = $('#selectModal');
      $.ajax({
        url: url,
        type: 'GET',
        cache: false
      }).done(function (html) {
        $modal.html(html);
        if (self.selectMode !== 'page') {
          $modal.find('#openLinkButton').hide();
        }
        $modal.modal('show');
      });
      return false;
    },
    applySelected: function (path, text, filesize, addPrefix) {
      var self = this;

      if (addPrefix) {
        path = '/' + self.filesPrefix + path;
      }

      if (self.selectText) {
        $('#' + self.selectText).val(text);
      }
      if (self.selectFilesize) {
        $('#' + self.selectFilesize).val(filesize);
      }
      if (self.selectHref) {
        $('#' + self.selectHref).val(path);
      } else if (self.selectSrc) {
        $('#' + self.selectSrc).val(path).trigger('change');
      }
      $('#selectModal').modal('hide');
      self.updated = true;
    },
    // 画像
    updateSrc: function (e) {
      var $input = $(e.target), id, src = '';

      id = $input.attr('data-img-src');
      if ($input.val()) {
        src = $input.val();
        $('[data-img="' + id + '"]').attr('src', src);
      } else {
        $('[data-img="' + id + '"]').attr('src', '');
      }
    },
    screen: function (e) {
      var $previewBox, $iframe, $target, width, height;

      $previewBox = $('#previewBox');
      $iframe = $('#previewIframe');
      $target = $(e.currentTarget);
      width = $target.attr('data-width') || '100%';
      height = $target.attr('data-height') || '100%';

      $('[data-width]').removeClass('btn-warning');
      $target.addClass('btn-warning');
      $('#openPreviewPanel > i').replaceWith($target.children().clone());

      if (width === '100%' && height === '100%') {
        $previewBox.removeClass('bezel');
      } else {
        $previewBox.addClass('bezel');
      }

      $iframe.css({
        'width': width,
        'height': height
      });
      $('#rotateScreen').removeClass('btn-warning');
    },
    rotateScreen: function (e) {
      var $iframe, $button, $selected, temp, width, height;

      $iframe = $('#previewIframe');
      $button = $(e.currentTarget);
      $selected = $('.btn-warning[data-width]');
      width = $selected.attr('data-width') || '100%';
      height = $selected.attr('data-height') || '100%';

      if (!$button.is('.btn-warning')) {
        temp = width;
        width = height;
        height = temp;
      }
      $iframe.css({
        'width': width,
        'height': height
      });
      $button.toggleClass('btn-warning');
    },
    findFocus: function () {
      var self = this, $target, id, $repeatItems, i = 0;

      $target = $(':focus');
      if (!$target.is('input, select, textarea, [data-meeditor-instance]')) {
        self.focusId = null;
        return;
      }

      if ($target.attr('data-meeditor-instance')) {
        $target = $('[data-meeditor="' + $target.attr('data-meeditor-instance') + '"]');
      }
      if ($target.is('[data-id]')) {
        id = $target.attr('data-id');
      } else {
        id = $target.attr('id');
      }
      if (id && id.indexOf(':') > 0) {
        id = id.substring(0, id.indexOf(':'));
      }

      // リピート番号を取得
      self.repeatNo = null;
      $repeatItems = $('[data-id="' + id + '"]');
      if ($repeatItems.length > 1) {
        $repeatItems.each(function () {
          if (this == $target.get(0)) {
            self.repeatNo = i;
            return;
          }
          i++;
        });
      }

      self.focusId = id;
    },
    scrollPreview: function () {
      var self = this, previewOffset, top, iframeOffsetTop, $previewElement;

      if (!self.focusId) {
        return;
      }

      $previewElement = self.$iframe.contents().find('[data-id="' + self.focusId + '"]');
      if (!$previewElement.length) {
        return;
      }
      // 対象が２件以上ある場合で、リピート番号があれば一致するものに絞り込む
      if ($previewElement.length > 1 && self.repeatNo !== null) {
        $previewElement = $previewElement.eq(self.repeatNo);
      }
      $previewElement.addClass('selectableField');
      setTimeout(function () {
        $previewElement.removeClass('selectableField');
      }, 3000);
      previewOffset = $previewElement.offset();

      iframeOffsetTop = self.$iframe.offset().top;
      top = previewOffset.top - self.focusTop + iframeOffsetTop;
      self.$iframe.contents().scrollTop(top);
    },
    selectBlock: function (e) {
      var self = this, $target, $currentTarget;

      $target = $(e.target);
      $currentTarget = $(e.currentTarget);
      // リピートコンテナ内をクリックして伝播してきた場合はスルー
      if ($target.is('[data-repeat-container] *') && $currentTarget.is('[data-container] > *')) {
        return;
      }

      if (self.$block) {
        self.deselectBlock(e);
      }

      self.$block = $(e.currentTarget);
      self.openToolbox();
    },
    deselectBlock: function (e) {
      var self = this;

      if (self.$block) {
        self.$block = null;
      }
      $('[data-container] > .selected, [data-repeat-container] > .selected').removeClass('selected');
      $('#containerToolbox').hide();
      $('.add-field-box').fadeOut();
    },
    openToolbox: function () {
      var self = this, $toolbox, offset;
      if (self.$block === null) {
        return;
      }
      self.$block.addClass('selected');

      $toolbox = $('#containerToolbox');
      offset = self.$block.offset();
      offset.top += self.$block.outerHeight();
      offset.left += self.$block.outerWidth() - $toolbox.outerWidth() - 16;
      $toolbox.show().css({
        top: offset.top + 'px',
        left: offset.left + 'px'
      });
      $('.add-field-box').hide();
    },
    addField: function (e) {
      var self = this;

      if (self.$block.parents('[data-repeat-container]').length) {
        // リピートブロック
        self.addItem(e);
      } else {
        // コンテナブロック
        self.openAddFieldBox(e);
      }
    },
    up: function () {
      var self = this;

      if (self.$block === null || self.$block.is(':first')) {
        return false;
      }
      self.$block.after(self.$block.prev());
      self.updated = true;
      self.openToolbox();
    },
    down: function () {
      var self = this, $inputs;

      if (self.$block === null || self.$block.is(':last')) {
        return false;
      }
      self.$block.before(self.$block.next());
      self.updated = true;

      $inputs = self.$block.find('input, select, textarea');
      $inputs.eq($inputs.length - 1).focus();

      $('#downField').focus();
      self.openToolbox();
    },
    remove: function (e) {
      var self = this;
      var $block = self.$block;

      if (!e.shiftKey && !confirm('フィールドを削除しますか？')) {
        return;
      }
      $('#containerToolbox').hide();
      if ($block.next().length) {
        self.$block = $block.next();
        $block.remove();
        self.openToolbox();
      } else {
        self.deselectBlock(e);
        $block.remove();
      }
      self.updated = true;
    },
    keyDown: function (e) {
      var self = this;

      switch (e.keyCode) {
        case 37: // left
        case 38: // up
          self.up();
          break;
        case 39: // right
        case 40: // down
          self.down();
          break;
      }
      return true;
    },
    countContainersLength: function () {
      var self = this, $container;

      $('[data-container]').each(function () {
        $container = $(this);
        self.containersLength[$container.attr('data-container')] = $container.children().length;
      });
    },
    openAddFieldBox: function (e) {
      var self = this, $button, $containerField, toolBoxClicked, $addFieldBox, diff, toolbarButtonOffset,
          addFieldBoxOffset;

      $button = $(e.currentTarget);
      if ($button.is('#addField')) {
        // ツールボックスのボタンをクリックした場合
        $containerField = self.$block.parents('[data-container-field]');
        toolBoxClicked = true;
      } else {
        $containerField = $button.parents('[data-container-field]');
        toolBoxClicked = false;
      }

      $addFieldBox = $('[data-add-field-box="' + $containerField.attr('data-container-field') + '"]');

      if ($addFieldBox.is(':visible')) {
        //$addFieldBox.hide();
        $addFieldBox.fadeOut();
        return;
      }
      if (toolBoxClicked) {
        toolbarButtonOffset = $button.offset();
        addFieldBoxOffset = $addFieldBox.parent().offset();
        diff = addFieldBoxOffset.top - toolbarButtonOffset.top + 18;
        $addFieldBox.css('margin-bottom', diff + 'px');
      } else {
        self.deselectBlock(e);
        $addFieldBox.css('margin-bottom', '0');
      }

      $addFieldBox.fadeIn(400, function () {
        var top = $addFieldBox.offset().top;
        var marginBottom = parseInt($addFieldBox.css('margin-bottom'), 10);
        if (top < self.headerHeight) {
          marginBottom = marginBottom - (self.headerHeight - top);
          $addFieldBox.css('margin-bottom', marginBottom + 'px');
        }
      });

      e.stopPropagation();
    },
    getField: function (e) {
      var self = this, $container, number, $block, $newBlock, $input, name, parentField, $button, $addFieldBox;

      $button = $(e.target);
      name = $button.attr('data-add-field-name');
      parentField = $button.attr('data-field-name');
      $addFieldBox = $button.parents('.add-field-box');
      $addFieldBox.hide();
      $container = $('[data-container="' + $addFieldBox.attr('data-add-field-box') + '"]');
      number = ++self.containersLength[parentField];
      $block = self.$block;
      if ($block && !$block.parent().is($container)) {
        $block = null;
      }

      $.ajax({
        url: '/admin/field/field/' + name + '/' + parentField + '/' + number,
        type: 'GET',
        cache: false,
        success: function (html) {
          $newBlock = $(html);
          if ($block) {
            $block.after($newBlock);
          } else {
            $container.append($newBlock);
          }
          if (window.hasOwnProperty('meeditor')) {
            window.meeditor.enableEditor();
          }
          $input = $newBlock.find('[data-meeditor-instance]');
          if ($input.length) {
            setTimeout(function () {
              $input.eq(0).click().focus();
            }, 0);
          } else {
            $newBlock.find('input, textarea, select').eq(0).click().focus();
          }
          mecms.datePicker();
          self.updated = true;
        }
      });
    },
    // プレビュー
    preview: function () {
      var self = this, $iframeBody, $iframe, doc, previewTop;

      self.updated = false;
      if (self.$iframe) {
        previewTop = self.$iframe.contents().scrollTop();
      }

      self.submitForm();
      $.ajax({
        url: '/admin/' + self.baseType + '/content/' + self.type + '/preview',
        type: 'POST',
        cache: false,
        data: $('#contentForm').serialize(),
        success: function (html) {
          self.$iframe = $iframe = $('#previewIframe');
          doc = $iframe.contents()[0];
          doc.open();
          doc.write(html);
          doc.close();

          $iframe.contents().scrollTop(previewTop);
          self.scrollPreview();

          $iframeBody = $(doc.body);
          $iframeBody.on('click', function (e) {
            if (e.altKey) {
              self.openPreviewPanel();
            } else {
              self.closeScreenSwitcher();
            }
            return false;
          });
          $iframeBody.on('click', '[data-id]', function (e) {
            self.deselectBlock(e);
            self.focusInput(e);
            return false;
          });
          $iframeBody.append('<style>' +
              '[data-id] {cursor:pointer;}' +
              '[data-id]:hover, .selectableField {box-shadow: 0 0 0 1px #3366FF inset, 0 0 1px 2px #99CCFF;}' +
              '</style>'
          );
          $iframeBody.on('touchstart', function (e) {
            $iframeBody.find('[data-id]').addClass('selectableField');
          });
          $iframeBody.on('touchend', function (e) {
            $iframeBody.find('[data-id]').removeClass('selectableField');
          });
        }
      });
    },
    focusInput: function (e) {
      var self = this, $target, id, inputOffset, $input, diff, $formPanelBody, $repeatItems, i = 0, repeatNo = null;

      // クリックされたプレビュー側の要素を取得
      $target = $(e.currentTarget);
      id = $target.attr('data-id');

      // idに一致する要素が複数ある場合は選択された要素のリピート番号を取得する
      $repeatItems = self.$iframe.contents().find('[data-id="' + id + '"]');
      if ($repeatItems.length > 1) {
        $repeatItems.each(function () {
          if (this == $target.get(0)) {
            repeatNo = i;
          }
          i++;
        });
      }

      if ($('#formPanelBody').is(':hidden')) {
        self.openPreviewPanel();
      }

      // フォーム側
      $input = $('#' + id);
      if ($('[data-repeat-container="' + id + '"]').length) {
        $input = $('[data-repeat-container="' + id + '"]');
      } else if (!$input.length) {
        $input = $('[data-id="' + id + '"]');
      }
      // 対象要素が複数あり、リピート番号がある場合は該当要素に絞り込む
      if ($input.length > 2 && repeatNo !== null) {
        $input = $input.eq(repeatNo);
      }
      if ($input.is('[data-meeditor]')) {
        $input = $('[data-meeditor-instance="' + $input.attr('data-meeditor') + '"]');
      }

      inputOffset = $input.offset();
      diff = inputOffset.top - self.focusTop;
      $formPanelBody = $('#formPanelBody');
      $formPanelBody.scrollTop($formPanelBody.scrollTop() + diff);

      if ($input.length > 1 && $input.is('[type="checkbox"]')) {
        $input.each(function () {
          if ($(this).is(':checked')) {
            $(this).focus();
          }
        });
      } else {
        if (!$input.is('input, textarea:visible, select, [data-meeditor-instance]')) {
          $input = $input.find('input, textarea:visible, select, [data-meeditor-instance]');
        }
        $input.eq(0).focus();
      }
    },
    submitForm: function () {
      var self = this;

      self.collectRepeatItem();
    },
    addItem: function (e) {
      var self = this, $button, fieldId, $repeatContainer, $itemContainer, toolBoxClicked;

      $button = $(e.currentTarget);
      // ブロック操作ツールボックスの場合
      if ($button.is('#addField')) {
        fieldId = self.$block.parents('[data-repeat-container]').attr('data-repeat-container');
        toolBoxClicked = true;
      } else {
        fieldId = $button.attr('data-add-item');
        toolBoxClicked = false;
      }
      $itemContainer = $('[data-item-container="' + fieldId + '"]');
      $repeatContainer = $('[data-repeat-container="' + fieldId + '"]');

      if (toolBoxClicked) {
        self.$block.after($itemContainer.children().clone());
        self.updated = true;
      } else {
        $repeatContainer.append($itemContainer.children().clone());
      }

      if ($itemContainer.find('[data-meeditor]').length && window.hasOwnProperty('meeditor')) {
        window.meeditor.enableEditor();
      }
    },
    collectRepeatItem: function () {
      var self = this, fieldId, $repeatContainer, $input, data = [], cols = {};

      $('[data-repeat-container]').each(function () {
        $repeatContainer = $(this);
        fieldId = $repeatContainer.attr('data-repeat-container');
        $repeatContainer.children().each(function () {
          $(this).find('input, textarea, select').each(function () {
            $input = $(this);
            cols[$input.attr('name')] = $input.val();
          });
          data.push(cols);
          cols = {};
        });
        $('#' + fieldId).val(JSON.stringify(data));
        data = [];
      });
    }
  };

})(jQuery, window);
