/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

(function ($, window) {
  'use strict';

  if (typeof window.mecms === 'undefined') {
    window.mecms = {};
  }

  /**
   * Portlet
   */
  mecms.portlet = {
    timeStamp: null,
    init: function () {
      var self = this;

      $('.column').sortable({
        connectWith: '.column',
        handle: '.card-header',
        cancel: '.portlet-toggle',
        tolerance: 'pointer',
        placeholder: 'portlet-placeholder ui-corner-all',
        cursorAt: {left: 10, top: 10},
        update: function (event, ui) {
          self.storePortlet(event)
        },
        over: function (event, ui) {
          $(event.target).parent().addClass('active');
        },
        out: function (event, ui) {
          $(event.target).parent().removeClass('active');
        }
      });
      $('[data-portlet]')
          .addClass('ui-widget ui-widget-content ui-helper-clearfix ui-corner-all')
          .children('.card-header')
          .addClass('ui-widget-header ui-corner-all')
          .prepend('<span class="ui-icon ui-icon-minusthick portlet-toggle"></span>');

      $('.portlet-toggle').on('click', function () {
        var icon = $(this);
        icon.toggleClass('ui-icon-minusthick ui-icon-plusthick');
        icon.closest('[data-portlet]').find('.card-body').toggle();
      }).click();
    },
    storePortlet: function (event) {
      var self = this, fd, $form, $balloon;

      if (self.timeStamp === event.timeStamp) {
        return;
      }
      self.timeStamp = event.timeStamp;

      $form = $('#portletForm');
      $balloon = $('.balloon-alert');

      fd = new FormData();
      fd.append('_METHOD', 'PUT');
      fd.append('csrf_name', $form.find('[name="csrf_name"]').val());
      fd.append('csrf_value', $form.find('[name="csrf_value"]').val());
      $('#top-column').find('[data-portlet]').each(function () {
        fd.append('top[]', $(this).attr('data-portlet'));
      });
      $('#left-column').find('[data-portlet]').each(function () {
        fd.append('left[]', $(this).attr('data-portlet'));
      });
      $('#right-column').find('[data-portlet]').each(function () {
        fd.append('right[]', $(this).attr('data-portlet'));
      });
      $('#bottom-column').find('[data-portlet]').each(function () {
        fd.append('bottom[]', $(this).attr('data-portlet'));
      });

      $.ajax({
        url: $form.attr('action'),
        type: 'POST',
        data: fd,
        cache: false,
        processData: false,
        contentType: false
      }).done(function (html) {
        if (!$balloon.children().is(':visible')) {
          $balloon.html(html).fadeIn(500).delay(2000).fadeOut(1000);
        }
      }).fail(function (jqXHR, textStatus, errorThrown) {
        $balloon.html(jqXHR.responseText).show().delay(4000).fadeOut(1000);
      });
    }
  };

})(jQuery, window);
