/*!
 * MicroEngine Editor
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

(function ($, window) {
  'use strict';

  var meeditor = {
    meeditorId: 0,
    onToolbar: false,
    $editor: null,
    range: null,
    isMSIE: false,
    init: function () {
      var self = this, $toolbar, $linkBox;

      self.detectUserAgent();
      self.enableEditor();
      $(document).on('focus', '[data-meeditor-instance]', function (e) {
        self.openToolbar(e);
      });
      $(document).on('blur', '[data-meeditor-instance]', function (e) {
        return self.closeToolbar(e);
      });
      document.addEventListener('scroll', function (e) {
        self.positionToolbar();
      }, true);
      $(document).on('click', function (e) {
        self.clickScreen(e);
      });
      $(document).on('keydown', '[data-meeditor-instance]', function (e) {
        return self.keydown(e);
      });
      $(document).on('keyup touchend', '[data-meeditor-instance]', function (e) {
        return self.updateButtonState(e);
      });
      $(document).on('paste', '[data-meeditor-instance]', function (e) {
        self.paste(e);
      });

      $toolbar = $('<div id="meeditorToolbar" class="meeditor-toolbar" style="display:none;">'
          + '<button type="button" id="meeditorBold" class="btn btn-sm btn-light" title="太字">'
          + '<i class="fa fa-bold"></i></button>'
          + '<button type="button" id="meeditorLink" class="btn btn-sm btn-light" title="リンク">'
          + '<i class="fa fa-link"></i></button>'
          + '<button type="button" id="meeditorCode" class="btn btn-sm btn-light" title="コード編集">'
          + '<i class="fa fa-code"></i></button>'
          + '</div>'
      );
      $('body').append($toolbar);
      $toolbar.on('mousedown', function (e) {
        self.saveRange(e);
      });
      $toolbar.on('mouseup', function (e) {
        self.reselect();
      });
      $(document).on('mouseover', '.meeditor-toolbar, .meeditor-popup', function (e) {
        self.onToolbar = true;
      });
      $(document).on('mouseout', '.meeditor-toolbar, .meeditor-popup', function (e) {
        self.onToolbar = false;
      });
      $toolbar.find('#meeditorBold').on('click', function (e) {
        self.bold(e);
      });
      $toolbar.find('#meeditorLink').on('click', function (e) {
        self.openLinkBox();
      });
      $linkBox = $('<div id="meeditorLinkBox" class="meeditor-popup" style="display:none;">'
          + '<div class="input-group input-group-sm">'
          + '<input type="text" id="toolbar-url" name="toolbar-url" class="form-control" value="" title="URL" placeholder="URL">'
          + '<div class="input-group-append">'
          + '<button class="btn btn-primary" data-select-mode="page" data-select-href="toolbar-url" data-select-text="toolbar-link-text">選択</button>'
          + '</div>'
          + '</div>'
          + '<input class="form-control form-control-sm" id="toolbar-link-text" name="toolbar-link-text" type="text" value="" title="リンクテキスト" placeholder="リンクテキスト">'
          + '<div class="form-check custom-control custom-checkbox">'
          + '<label class="form-check-label">'
          + '<input class="form-check-input" type="checkbox" name="toolbar-target">新しいタブで開く'
          + '</label>'
          + '</div>'
          + '<div class="meeditor-popup-footer">'
          + '<button id="meLinkCancel" class="btn btn-sm btn-dark">キャンセル</button>'
          + '<button id="meLinkApply" class="btn btn-sm btn-success">OK</button>'
          + '<button id="meLinkUnlink" class="btn btn-sm btn-danger pull-right">解除</button>'
          + '</div>'
          + '</div>'
      );
      $('body').append($linkBox);
      $linkBox.find('#meLinkCancel').on('click', function (e) {
        self.closeLinkBox(e);
      });
      $linkBox.find('#meLinkApply').on('click', function (e) {
        self.applyLink(e);
      });
      $linkBox.find('#meLinkUnlink').on('click', function (e) {
        self.unlink(e);
      });
      $(document).on('submit', function (e) {
        self.saveEditor();
      });
    },
    detectUserAgent: function () {
      var self = this;

      if (navigator.userAgent.indexOf('MSIE') !== -1 || navigator.userAgent.indexOf('Trident') !== -1) {
        self.isMSIE = true;
      }
    },
    enableEditor: function () {
      var self = this, $textareaList, $textarea, $editor, id;

      $textareaList = $('[data-meeditor=""]');
      $textareaList.each(function () {
        $textarea = $(this);
        if ($textarea.parents('[data-item-container]').length) {
          // 追加用の隠し要素は除外する
          return;
        }
        id = ++self.meeditorId;
        $textarea.attr('data-meeditor', id);

        $editor = $('<div class="meeditor" contenteditable="true"></div>');
        $editor.html($textarea.val());
        $editor.attr('data-meeditor-instance', id);
        $editor.css({
          'min-height': $textarea.height() + 'px',
          'padding-top': $textarea.css('padding-top'),
          'padding-right': $textarea.css('padding-right'),
          'padding-bottom': $textarea.css('padding-bottom'),
          'padding-left': $textarea.css('padding-left')
        });

        $textarea.hide();
        $textarea.after($editor);
      });

      document.execCommand('DefaultParagraphSeparator', false, 'br');
    },
    openToolbar: function (e) {
      var self = this, $editor, $codeButton;

      $editor = $(e.target);
      if (!$editor.is('[data-meeditor-instance]')) {
        $editor = $('[data-meeditor-instance]:focus');
        if ($editor.length < 1) {
          self.$editor = null;
          return;
        }
      }
      $codeButton = $('#meeditorCode');
      $codeButton.attr('data-code', '[data-meeditor-instance="' + $editor.attr('data-meeditor-instance') + '"]');

      self.$editor = $editor;
      $('#meeditorToolbar').show();
      $('.meeditor-popup').hide();
      self.positionToolbar();
      setTimeout(function () {
        self.positionToolbar();
      }, 10);
    },
    closeToolbar: function (e) {
      var self = this;

      if (self.onToolbar) {
        return;
      }
      self.saveToTextarea(e);
      self.closePopup();
    },
    positionToolbar: function () {
      var self = this, $editor, $toolbar, offset;

      $editor = self.$editor;
      $toolbar = $('#meeditorToolbar');
      if (!$editor || $toolbar.is(':hidden')) {
        return;
      }

      offset = $editor.offset();
      offset.top -= $toolbar.outerHeight();
      $toolbar.show().css({
        top: offset.top + 'px',
        left: offset.left + 'px'
      });
      self.positionLinkBox();
    },
    closePopup: function () {
      var self = this;

      $('.meeditor-toolbar, .meeditor-popup').hide();
      self.range = null;
    },
    /**
     * コマンド実行
     */
    execCommand: function (commandName, showDefaultUI, valueArgument) {
      var self = this;

      if (showDefaultUI === undefined) {
        showDefaultUI = false;
      }
      if (valueArgument === undefined) {
        valueArgument = null;
      }
      return document.execCommand(commandName, showDefaultUI, valueArgument);
    },
    /**
     * 太字
     */
    bold: function (e) {
      this.execCommand('bold', false, null);
    },
    /**
     * リンク作成ポップアップを開く
     */
    openLinkBox: function () {
      var self = this, $a, $url, $text, $target, $unlinkButton, range;

      $url = $('#meeditorLinkBox [name=toolbar-url]');
      $text = $('#meeditorLinkBox [name=toolbar-link-text]');
      $target = $('#meeditorLinkBox [name=toolbar-target]');
      $unlinkButton = $('#meLinkUnlink');
      $a = self.anchorAtCursor();
      if ($a) {
        // 再編集
        $url.val($a.attr('href'));
        $text.hide();
        $target.prop('checked', $a.prop('target'));
        $unlinkButton.show();
      } else {
        // 新規
        $url.val('');

        range = self.getRange();
        if (range && range.collapsed) {
          $text.show().val('');
        } else {
          $text.hide();
        }
        $target.prop('checked', false);
        $unlinkButton.hide();
      }
      $('#meeditorLinkBox').show();
      self.positionLinkBox();
      $url.focus();
    },
    positionLinkBox: function () {
      var $toolbar, $linkBox, offset;

      $linkBox = $('#meeditorLinkBox');
      if ($linkBox.is(':hidden')) {
        return;
      }

      $toolbar = $('#meeditorToolbar');
      offset = $toolbar.offset();
      offset.top += $toolbar.outerHeight();
      $linkBox.show().css({
        top: offset.top + 'px',
        left: offset.left + 'px'
      });
    },
    /**
     * リンクを適用する
     */
    applyLink: function (e) {
      var self = this, url, $text, text, linkTextEnabled, target, range, $a, anchor;

      url = $('#meeditorLinkBox [name=toolbar-url]').val();
      $text = $('#meeditorLinkBox [name=toolbar-link-text]');
      text = $text.val();
      linkTextEnabled = $text.is(':visible');

      if (url === '') {
        alert('URLを入力してください。');
        return;
      }

      if (linkTextEnabled && text === '') {
        text = url;
      }
      target = $('#meeditorLinkBox [name=toolbar-target]').prop('checked');
      self.reselect();

      // 親のA要素を取得
      $a = self.anchorAtCursor();

      if ($a) {
        // 再編集
        $a.attr('href', url);
        self.updateTarget($a, target);
      } else {
        // 新規
        if (linkTextEnabled) {
          // 範囲選択なし
          $a = $('<a href=""></a>');
          $a.attr('href', url);
          $a.text(text);

          range = self.getRange();
          if (range) {
            anchor = $a.get(0);
            range.insertNode(anchor);
            self.cursorAfter(anchor);
            self.updateTarget($a, target);
          }
        } else {
          // 範囲選択あり
          self.execCommand('createLink', false, url);

          $a = self.anchorAtCursor();
          if ($a) {
            // 選択範囲の先祖のA要素を取得しなおしてtargetを更新
            self.updateTarget($a, target);
          }
        }
      }
      self.closeLinkBox(e);
    },
    /**
     * リンク解除
     */
    unlink: function (e) {
      var self = this, range, $a;

      self.reselect();
      range = self.getRange();
      if (range && range.collapsed) {
        // 選択範囲の先祖のA要素を取得
        $a = $(range.startContainer.parentNode).parents('a').addBack().eq(0);
        if ($a.is('a')) {
          // テキストも含めて $a の後に移動
          $a.after($a.contents());
          $a.remove();
        }
      } else {
        self.execCommand('unlink');
      }
      self.closeLinkBox(e);
    },
    closeLinkBox: function (e) {
      var self = this;

      $('#meeditorLinkBox').hide();
      self.openToolbar(e);
      self.reselect();
    },
    updateTarget: function ($a, target) {
      if ($a) {
        // target 設定
        if (target) {
          $a.attr('target', '_blank');
        } else {
          $a.removeAttr('target');
        }
      }
    },
    clickScreen: function (e) {
      var self = this, $target;

      $target = $(e.target);
      if ($target.is('[data-meeditor-instance]') || $target.parents('[data-meeditor-instance]').length > 0) {
        self.updateButtonState();
      } else if (!$target.is('.meeditor-toolbar, .meeditor-popup, [data-meeditor-instance], [data-meeditor]')
          && $target.parents('.meeditor-toolbar, .meeditor-popup, [data-meeditor-instance], .modal').length < 1) {
        self.closePopup();
      }
    },
    keydown: function (e) {
      var self = this, range, br;

      switch (e.keyCode) {
        case 13:
          // enter key
          if (self.isMSIE) {
            range = self.getRange();
            br = document.createElement('br');
            range.insertNode(br);

            if (self.isTerminated()) {
              // テキストノードの最後の場合は br を2個追加する。
              br = document.createElement('br');
              range.insertNode(br);
            }
            self.cursorAfter(br);

            return false;
          }
        default:
          break;
      }
      return true;
    },
    /**
     * キーアップイベント
     */
    keyup: function (e) {
      var self = this;

      self.updateButtonState();

      return true;
    },
    paste: function (e) {
      var self = this, text, range, textNode;

      if (e.originalEvent.clipboardData) {
        e.preventDefault();
        text = e.originalEvent.clipboardData.getData('text/plain');
        self.execCommand('insertHTML', false, text);
      } else if (window.clipboardData) {
        e.preventDefault();
        text = window.clipboardData.getData('Text');
        textNode = document.createTextNode(text);
        range = self.getRange();
        range.insertNode(textNode);
      }
    },
    /**
     * ツールバーボタンに適用状態を反映する
     */
    updateButtonState: function () {
      var self = this, selection, range, $parent, $button, buttons;

      buttons = {
        'link': {
          'selector': '#meeditorLink',
          'on': false
        },
        'bold': {
          'selector': '#meeditorBold',
          'on': false
        }
      };

      selection = window.getSelection();
      if (selection.focusNode && selection.focusNode.nodeType === 3) {
        range = self.getRange();
        if (range) {
          $parent = $(range.startContainer.parentNode);

          do {
            // bold 対応
            if (typeof buttons.bold !== 'undefined' && !buttons.bold.on && $parent.is('b, strong')) {
              buttons.bold.on = true;
            }
            // link 対応
            if (typeof buttons.link !== 'undefined' && !buttons.link.on && $parent.is('a')) {
              buttons.link.on = true;
            }
            $parent = $parent.parent();
            // ブロック部品もしくはブロック要素なら終了
          } while ($parent[0] && $parent.is('[data-meeditor-instance] *'));
        }
      }
      // 各ボタンに反映
      $.each(buttons, function (i, val) {
        $button = $(val.selector);
        if (val.on) {
          $button.addClass('active');
        } else {
          $button.removeClass('active');
        }
      });
    },
    /**
     * カーソル位置のA要素を取得
     */
    anchorAtCursor: function () {
      var self = this, $a, range;

      range = self.getRange();
      if (range) {
        // 選択範囲先頭の先祖のA要素を取得。
        $a = $(range.endContainer);
        if (!$a.is('a')) {
          $a = $a.parents('a').eq(0);
        }
      }
      if (!$a.is('a')) {
        $a = null;
      }
      return $a;
    },
    saveToTextarea: function (e) {
      var $editor, $textarea;

      $editor = $(e.target);
      $textarea = $('[data-meeditor="' + $editor.attr('data-meeditor-instance') + '"]');
      if ($textarea.val() !== $editor.html()) {
        $textarea.val($editor.html()).change();
      }
    },
    saveEditor: function () {
      var $editor, $textarea;

      $('[data-meeditor-instance]').each(function () {
        $editor = $(this);
        $textarea = $('[data-meeditor="' + $editor.attr('data-meeditor-instance') + '"]');
        $textarea.val($editor.html());
      });
    },
    isTerminated: function () {
      var self = this, range, re, terminated = false, next;

      range = self.getRange();

      if (range && self.isBlock(range.endContainer)) {
        // コンテナの親がブロックレベル要素の場合だけチェックする
        if (range.endContainer.childNodes.length == range.endOffset) {
          // カーソルがコンテナの終端にいる場合
          terminated = true;
        } else {
          // カーソルの次の要素を取得する
          next = range.endContainer.childNodes[range.endOffset];
          // スペース文字以外の正規表現
          re = /\S/;
          while (next) {
            if (next.nodeType == 1) {
              // 要素の場合は、ブロックかどうかで終端かどうかを判定する
              terminated = self.isBlock(next);
              break;
            } else if (next.nodeType == 3) {
              if (re.test(next.nodeValue)) {
                // テキストノードの場合、スペース文字以外の文字があれば終端ではない
                terminated = false;
                break;
              }
            }
            // スペース文字だけの場合は次の要素をチェック
            next = next.nextSibling;
            if (!next) {
              // 次の要素がなければ終端である
              terminated = true;
            }
          }
        }
      }
      return terminated
    },
    isBlock: function (node) {
      var blockList, display;

      if (node.nodeType == 3) {
        return false;
      }

      if (node.hasAttribute('data-meeditor-instance')) {
        return true;
      }

      blockList = ['block', 'table', 'table-cell', 'flex', 'grid'];
      display = getComputedStyle(node)['display'];
      if (blockList.indexOf(display) >= 0) {
        return true;
      }

      return false;
    },
    reselect: function () {
      var self = this, selection;

      if (self.range) {
        selection = window.getSelection();
        selection.removeAllRanges();
        selection.addRange(self.range);
      }
    },
    cursorAfter: function (node) {
      var self = this, range;

      range = self.getRange();
      if (!range) {
        return;
      }

      range.setStartAfter(node);
      range.setEndAfter(node);
      self.range = range;
      self.reselect();
    },
    saveRange: function (e) {
      var self = this, range;

      range = self.getRange();
      if (range) {
        self.range = range;
      }
    },
    getRange: function () {
      var selection, range = null;

      selection = window.getSelection();
      if (selection.rangeCount > 0) {
        range = selection.getRangeAt(0);
      }
      return range;
    }
  };

  window.meeditor = meeditor;
})(jQuery, window);
